package main

import (
	"errors"

	"go.wit.com/lib/protobuf/gitpb"
	"go.wit.com/log"
)

func doStrict(repo *gitpb.Repo) error {
	if !repo.IsValidDir() {
		log.Info(repo.GetGoPath(), "is invalid. fix your repos.pb file with 'forge' first")
		log.Info("")
		log.Info("go install go.wit.com/apps/forge@latest")
		log.Info("")
		return errors.New(repo.GetGoPath() + " is invalid. fix your repository list with 'forge' first")
	}
	log.Info(repo.GetGoPath(), "is valid according to forge")

	repo.Run([]string{"git", "notes", "remove"})

	// erase the go.mod and go.sum files
	eraseGoMod(repo)

	if repo.GetMasterBranchName() != repo.GetCurrentBranchName() {
		log.Info("")
		log.Info("You are not operating on your git master branch.")
		log.Info("Publishing go.mod & go.sum files must come from from git version tag on the master branch")
		log.Info("")
		return errors.New(repo.GetGoPath() + " not in the git master branch")
	}

	err := repo.RepoIgnoresGoMod()
	if err != nil {
		log.Info(repo.GetGoPath(), "some wierd git error happened. investigate.", err)
		return err
	}

	if forge.Config.IsReadOnly(repo.GetGoPath()) {
		log.Info("you can not push to read only repositories.", repo.GetGoPath())
		log.Info("change your .config/forge/ to indicate you own this repository")
		return nil
	}

	if repo.CheckDirty() {
		log.Info("")
		log.Info("You can not continue on a dirty git repo.")
		log.Info("")
		return errors.New(repo.GetGoPath() + " git repo is dirty")
	}

	log.Info(repo.GetGoPath(), "GOING TO MAKE NEW go.* FILES")

	// actually will re-create go.sum and go.mod now
	if err := redoGoMod(repo); err != nil {
		return err
	}

	// the first time, it'll attempt to fix some stuff
	cleanGoDepsCheckOk(repo)
	// try to trim junk
	if err := trimGoSum(repo); err != nil {
		return err
	}
	repo.ParseGoSum()

	// check go.sum file
	if err := cleanGoDepsCheckOk(repo); err != nil {
		log.Info("forge.FinalGoDepsCheck() failed. boo. :", repo.GetGoPath())
		return err
	}

	// put the files in the notes section in git
	// this way, git commits are not messed up
	// with this autogenerated code
	if err := saveAsMetadata(repo); err != nil {
		log.Info("save go.mod as git metadata failed", repo.GetGoPath(), err)
		return err
	}

	// everything worked!
	configSave = true
	return nil
}
